package app;

import java.io.*;
import java.util.*;
import javax.bluetooth.*;
import javax.microedition.io.*;
import javax.microedition.io.file.FileConnection;
import javax.microedition.lcdui.*;
import javax.microedition.lcdui.List;
import javax.obex.*;

public class BTList extends List implements CommandListener, DiscoveryListener
{

  public static int BUF_SIZE = 1024;
  public static int SLEEP_TIME = 10;

  private static Image imageBT;

  private Display display;
  private Displayable parent;

  private String name;
  private LocalDevice local;  
  private DiscoveryAgent agent;
  private boolean inquiryFinished = false;
  private Vector remotedevices = new Vector();
  private ServiceRecord[] servRecord;

  private Command commandOk = new Command("Ok", Command.OK, 0);
  private Command commandCancel = new Command("Cancel", Command.CANCEL, 0);
  
  static 
  {
    try
    {
      imageBT = Image.createImage("/res/bluetooth.png");
    }
    catch (IOException e)
    {
    }
  }
  
  public BTList(Display display, Displayable parent, String name) 
  {
    super("Devices:", List.IMPLICIT);
    this.display = display;
    this.parent = parent;
    this.name = name;
    addCommand(commandCancel);
    setCommandListener(this);
    startListener();
    display.setCurrent(this);
  }
  
  public static String extractName(String resource) 
  {
    String s = resource.trim();
    s = s.substring(s.lastIndexOf('/') + 1);
    return s;
  }  

  public static String formatKb(long i) 
  {
    if (i < 1000) return "" + i + " byte(s)"; 
    else if (i < 1000000) return "" + i / 1000 + " Kbyte(s)"; 
    else return "" + i / 1000000 + " Mbyte(s)"; 
  }
   
  private void startListener() 
  {
    try
    {
      local = LocalDevice.getLocalDevice();
      local.setDiscoverable(DiscoveryAgent.GIAC);
      agent = local.getDiscoveryAgent();
      startInquiry();
    }
    catch (BluetoothStateException e) 
    {
    }
  }

  private void stopListener() 
  {
    try
    {
      cancelInquiry();
      local.setDiscoverable(DiscoveryAgent.NOT_DISCOVERABLE);
    }
    catch (BluetoothStateException e) 
    {
    }
  }
  
  private void startInquiry() 
  {
    inquiryFinished = false;
    try
    {
      agent.startInquiry(DiscoveryAgent.GIAC, this);
    }
    catch (BluetoothStateException e) 
    {
    }
  }
  
  private void cancelInquiry() 
  {
    if (!inquiryFinished) agent.cancelInquiry(this);
  }

  private void searchServices(RemoteDevice btDevice) 
  {
    UUID[] uuidSet = {new UUID(0x1105)};    
    try
    {
      agent.searchServices(null, uuidSet, btDevice, this);
    }
    catch (IOException e) 
    {
    }
  }
  
  public void commandAction(Command c, Displayable d) 
  {
    if (c == commandCancel)  
    {
      stopListener();
      display.setCurrent(parent);
    } else if (c == commandOk) 
    {
      int index = getSelectedIndex();
      if (index != -1) searchServices((RemoteDevice)remotedevices.elementAt(index));
    }        
  }
  
  private void send(ClientSession conn) throws IOException 
  {
    FileConnection fc = (FileConnection)Connector.open("file://" + name, Connector.READ);
    int fileSize = (int)fc.fileSize();
    InputStream is = fc.openInputStream();
    ClientSession session = (ClientSession)conn;
    HeaderSet header = session.createHeaderSet();
    session.connect(header);
    header.setHeader(HeaderSet.NAME, extractName(name));
    header.setHeader(HeaderSet.LENGTH, new Long(fileSize));
    Operation operation = session.put(header);
    OutputStream os = operation.openOutputStream();            
    byte[] abyte = new byte[BUF_SIZE];
    int writed = 0;
    int dummy = 0;
    Form form = new Form(extractName(name));
    Gauge gauge = new Gauge("", false, Gauge.INDEFINITE, 0);
    form.append(gauge);
    display.setCurrent(form);
    while ((dummy = is.read(abyte, 0, abyte.length)) != -1) 
    {
      writed += dummy;
      os.write(abyte, 0, dummy);
      os.flush();
      gauge.setLabel(formatKb(writed) + " / " + formatKb(fileSize));
      gauge.setMaxValue(fileSize);
      gauge.setValue(writed);
      try
      {
        Thread.sleep(SLEEP_TIME);
      }
      catch (InterruptedException e)
      {
      }
    }
    os.close();
    operation.close();
    session.close();            
    fc.close();
    fc = null;
  }

  public void deviceDiscovered(RemoteDevice btDevice, DeviceClass cod) 
  {
    try
    {
      append(btDevice.getFriendlyName(false), imageBT);
      remotedevices.addElement(btDevice);
    }
    catch (Exception e) 
    {
    }
  }
  
  public void inquiryCompleted(int discType) 
  {
    inquiryFinished = true;
    setSelectCommand(commandOk);
  }
  
  public void servicesDiscovered(int transID, ServiceRecord[] servRecord) 
  {
    if (servRecord != null) 
    {   
      this.servRecord = servRecord;
      agent.cancelServiceSearch(transID);
    }
  }
  
  public void serviceSearchCompleted(int transID, int respCode) 
  {
    if (servRecord != null) 
    {
      try
      {
        for (int i = 0; i < servRecord.length; i++) 
        {
          String url = (servRecord[i].getConnectionURL(ServiceRecord.NOAUTHENTICATE_NOENCRYPT, false));
          Connection conn = Connector.open(url);
          if (conn instanceof ClientSession) 
          {
            send((ClientSession)conn);
            conn.close();
            break;
          } else conn.close();
        }      
        display.setCurrent(parent);
      }
      catch (IOException e) 
      {
        Alert alert = new Alert("IOException");
        alert.setString("" + e.getMessage());
        alert.setTimeout(Alert.FOREVER);
        display.setCurrent(alert, parent);
      }
    }
  }  

}
